@@ -14,6 +14,7 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
+
 from gi.repository import Gtk, Gdk, GdkPixbuf, Champlain
 
 from maps.searcher import Searcher, Guide, ResultView
@@ -169,7 +170,12 @@ class MapWindow(Gtk.ApplicationWindow):
 
         searchview = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         searchview.pack_start(searchheader, False, False, 0)
-        searchview.pack_end(searchlist, True, True, 0)
+
+        swH_search = Gtk.ScrolledWindow() ## ADD THE SCROLLBAR
+        swH_search.set_vexpand(False)
+        swH_search.add(searchlist)
+        searchview.pack_end(swH_search, True, True, 0)
+
         vpaned.pack1(searchview)
 
         ############
@@ -197,7 +203,10 @@ class MapWindow(Gtk.ApplicationWindow):
         directionlist.append_column(column_time)
         directionlist.append_column(column_distance)
 
-        searchview.pack_end(directionlist, True, True, 0)
+        swH_direction = Gtk.ScrolledWindow() ## ADD THE SCROLLBAR
+        swH_direction.set_vexpand(False)
+        swH_direction.add(directionlist)
+        searchview.pack_end(swH_direction, True, True, 0)
 
         ########
         # Places
@@ -212,6 +221,17 @@ class MapWindow(Gtk.ApplicationWindow):
         column_name = Gtk.TreeViewColumn("Places", render_name, text=0)
         placelist.append_column(column_name)
 
+        ########
+        # Tracks
+        ########
+
+        self.trackstore = Gtk.ListStore(str, bool, object)  # name, boolean, track
+        tracklist = Gtk.TreeView(self.trackstore)
+
+        render_name = Gtk.CellRendererText()
+        column_name = Gtk.TreeViewColumn("Tracks", render_name, text=0)
+        tracklist.append_column(column_name)
+
         # Other
 
         self.sidebar = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
@@ -219,6 +239,7 @@ class MapWindow(Gtk.ApplicationWindow):
         markerbutton = Gtk.Button(label="Mark place")
         self.sidebar.pack_start(markerbutton, False, False, 10)
         self.sidebar.pack_start(placelist, False, False, 0)
+        self.sidebar.pack_start(tracklist, False, False, 0)
 
         hpaned.pack2(self.sidebar, False, False)
 
@@ -262,8 +283,16 @@ class MapWindow(Gtk.ApplicationWindow):
             #self.store[path][2].show()
 
     def on_drag_data_received(self, window, context, x, y, data, info, time):
-        g = GpxTrack(self.view, self.store)
-        g.import_file(data.get_text().strip())
+        filename = data.get_text().strip()
+        track = GpxTrack(self.view, self.trackstore)
+        try:
+            track.import_file(filename)
+        except:
+            print "Import error, could not find %s" % filename
+        else:
+            track.show()
+            track.go_to()
+            self.sidebartogglebutton.set_active(True)
 
     def on_result_clicked(self, selection):
         model, treeiter = selection.get_selected()
@@ -401,7 +430,7 @@ class MapWindow(Gtk.ApplicationWindow):
         marker_list.set_cursor(0, column=marker_list.get_column(0), start_editing=True)
 
     def on_marker_edited(self, cell, path, text, model):
-        if path != None:
+        if path != None and text != "":
             model[path][1].set_text(text)
             model[path][0] = text
 
@@ -415,11 +444,16 @@ class MapWindow(Gtk.ApplicationWindow):
 
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
-            track = GpxTrack(self.view, self.store)
-            track.import_file(dialog.get_filename())
-            track.show()
-            track.go_to()
-            self.sidebartogglebutton.set_active(True)
+            filename = dialog.get_filename()
+            track = GpxTrack(self.view, self.trackstore)
+            try:
+                track.import_file(filename)
+            except:
+                print "Import error, could not find %s" % filename
+            else:
+                track.show()
+                track.go_to()
+                self.sidebartogglebutton.set_active(True)
 
         elif response == Gtk.ResponseType.CANCEL:
             print "Cancel clicked"
@@ -431,7 +465,7 @@ class MapWindow(Gtk.ApplicationWindow):
         searcher = Searcher(marker_layer, resultview.liststore)
         text = searchfield.get_text()
         searcher.search(text)
-        resultview.show("Results for: %s" % text)
+        resultview.show("Results for: %s" % text, 2) ## 2 -> show search results
 
         marker_layer.show()
 
@@ -442,23 +476,23 @@ class MapWindow(Gtk.ApplicationWindow):
             error = guide.search(fromfield.get_marker(), tofield.get_marker())
             if not error:
                 marker_layer.show()
-                resultview.show("Directions")
+                resultview.show("Directions", 1) ## 1 -> show directions
                 directionlist.show()
             else:
-                resultview.show(error)
+                resultview.show(error, 1) ## 1 -> show directions
         else:
             marker_layer = resultview.marker_layer
             if tofield.get_text() and not tofield.get_marker():
                 searcher = Searcher(marker_layer, resultview.liststore)
                 text = tofield.get_text()
                 searcher.search(text)
-                resultview.show("Possible destination locations named: %s" % text)
+                resultview.show("Possible destination locations named: %s" % text, 2) ## 2 -> show search results
                 marker_layer.show()
             elif fromfield.get_text() and not fromfield.get_marker():
                 searcher = Searcher(marker_layer, resultview.liststore)
                 text = fromfield.get_text()
                 searcher.search(text)
-                resultview.show("Possible departure locations named: %s" % text)
+                resultview.show("Possible departure locations named: %s" % text, 2) ## 2 -> show search results
                 marker_layer.show()
             else:
                 print "Invalid search"
@@ -467,7 +501,7 @@ class MapWindow(Gtk.ApplicationWindow):
         guide = Guide(self.application, marker_layer, directionlist.get_model())
         guide.search(1,1)
         marker_layer.show()
-        resultview.show("Directions")
+        resultview.show("Directions", 1) ## 1 -> show directions
         directionlist.show()
 
     def add_filters(self, dialog):
